﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Analysis;
using HeuristicLab.Common;
using HeuristicLab.Problems.DataAnalysis;
using HeuristicLab.Visualization.ChartControlsExtensions;

namespace HeuristicLab.DataPreprocessing {

  public abstract class ScatterPlotContent : PreprocessingChartContent {
    protected ScatterPlotContent(IFilteredPreprocessingData preprocessingData)
      : base(preprocessingData) {
    }

    protected ScatterPlotContent(ScatterPlotContent content, Cloner cloner)
      : base(content, cloner) {
    }

    public ScatterPlot CreateScatterPlot(string variableNameX, string variableNameY, string variableNameGroup = "-") {
      ScatterPlot scatterPlot = new ScatterPlot();

      IList<double> xValues = PreprocessingData.GetValues<double>(PreprocessingData.GetColumnIndex(variableNameX)).Where(x => !double.IsNaN(x) && !double.IsInfinity(x)).ToList();
      IList<double> yValues = PreprocessingData.GetValues<double>(PreprocessingData.GetColumnIndex(variableNameY)).Where(x => !double.IsNaN(x) && !double.IsInfinity(x)).ToList();

      if (xValues.Any()) {
        try {
          double axisMin, axisMax, axisInterval;
          ChartUtil.CalculateOptimalAxisInterval(xValues.Min(), xValues.Max(), out axisMin, out axisMax, out axisInterval);
          scatterPlot.VisualProperties.XAxisMinimumAuto = false;
          scatterPlot.VisualProperties.XAxisMaximumAuto = false;
          scatterPlot.VisualProperties.XAxisMinimumFixedValue = axisMin;
          scatterPlot.VisualProperties.XAxisMaximumFixedValue = axisMax;
        } catch (ArgumentOutOfRangeException) { } // error during CalculateOptimalAxisInterval 
      }
      if (yValues.Any()) {
        try {
          double axisMin, axisMax, axisInterval;
          ChartUtil.CalculateOptimalAxisInterval(yValues.Min(), yValues.Max(), out axisMin, out axisMax, out axisInterval);
          scatterPlot.VisualProperties.YAxisMinimumAuto = false;
          scatterPlot.VisualProperties.YAxisMaximumAuto = false;
          scatterPlot.VisualProperties.YAxisMinimumFixedValue = axisMin;
          scatterPlot.VisualProperties.YAxisMaximumFixedValue = axisMax;
        } catch (ArgumentOutOfRangeException) { } // error during CalculateOptimalAxisInterval 
      }

      if (variableNameGroup == null || variableNameGroup == "-") {
        List<Point2D<double>> points = new List<Point2D<double>>();

        for (int i = 0; i < xValues.Count; i++) {
          Point2D<double> point = new Point2D<double>(xValues[i], yValues[i]);
          points.Add(point);
        }

        ScatterPlotDataRow scdr = new ScatterPlotDataRow(variableNameX + " - " + variableNameY, "", points);
        scdr.VisualProperties.IsVisibleInLegend = false;
        scatterPlot.Rows.Add(scdr);

      } else {
        var groupValues = PreprocessingData.GetValues<double>(PreprocessingData.GetColumnIndex(variableNameGroup));
        var data = xValues.Zip(yValues, (x, y) => new { x, y }).Zip(groupValues, (v, c) => new { v.x, v.y, c }).ToList();
        foreach (var groupValue in groupValues.Distinct()) {
          var values = data.Where(x => x.c == groupValue);
          var row = new ScatterPlotDataRow(
            variableNameGroup + " (" + groupValue + ")",
            "",
            values.Select(v => new Point2D<double>(v.x, v.y))) {
            VisualProperties = { PointSize = 6 }
          };
          scatterPlot.Rows.Add(row);
        }
      }
      return scatterPlot;
    }

    public DataRow GetCorrelationRow(string variableNameX, string variableNameY) {
      var xValues = PreprocessingData.GetValues<double>(PreprocessingData.GetColumnIndex(variableNameX));
      var yValues = PreprocessingData.GetValues<double>(PreprocessingData.GetColumnIndex(variableNameY));

      double k, d;
      OnlineCalculatorError err;
      OnlineLinearScalingParameterCalculator.Calculate(xValues, yValues, out k, out d, out err);
      double p = OnlinePearsonsRCalculator.Calculate(xValues, yValues, out err);

      var data = new double[xValues.Count];
      for (int i = 0; i < xValues.Count; i++) {
        data[i]= k * i + d;
      }

      return new DataRow(string.Format("Correlation (R²={0})", p*p), "", data);
    }
  }
}
